"""AI-Parrot Tools Meta information."""

__title__ = "ai-parrot-tools"
__description__ = "Tools and toolkits for AI-Parrot agents."
__version__ = "0.1.4"
__author__ = "Jesus Lara"
__author_email__ = "jesuslara@phenobarbital.info"
__license__ = "MIT"
__copyright__ = "Copyright (c) 2024-2026 Jesus Lara"
